local AceOO = AceLibrary("AceOO-2.0")

ShakedownAddonBase = AceOO.Class("AceDebug-2.0")

-- this is an abstract class, that all addon plugins should inherit from
ShakedownAddonBase.virtual = true

-- data members
ShakedownAddonBase.prototype.addonName = ""
ShakedownAddonBase.prototype.displayName = ""
ShakedownAddonBase.prototype.enabled = true
ShakedownAddonBase.prototype.activated = false
ShakedownAddonBase.prototype.loaded = false

-- table to hold each of the addons
SDAddons = {}

-- constructor
function ShakedownAddonBase.prototype:init(addonName, displayName, alreadyLoaded)
    
    ShakedownAddonBase.super.prototype.init(self)
    
    -- set the names
    self.displayName = displayName
    self.addonName = addonName
    
    if alreadyLoaded then
        
        -- addon is ready to go
        self.loaded = true
        table.insert(SDAddons, self)
        
    else
    
        -- is this addon loaded?
        self.loaded = (IsAddOnLoaded(addonName) == 1)
        
        if self.loaded then

            -- add to the list
            table.insert(SDAddons, self)
            
        else
            -- check if this is LOD
            local LOD = IsAddOnLoadOnDemand(addonName)
            
            -- check if this is enabled
            local _, _, _, enabled, loadable, reason = GetAddOnInfo(addonName)
            
            -- see if the addon is enabled and loadable
            if enabled == 1 and (loadable == 1 or reason == "NOT_DEMAND_LOADED" or reason == "DEP_NOT_DEMAND_LOADED") then
            
                if LOD ~= 1 then
                
                    -- addon is not a blizz LOD addon, so just call it loaded as it will load itself later
                    self.loaded = true
                
                end
                
                table.insert(SDAddons, self)
                
            end
            
        end
        
    end

end

-- load an LOD addon, returning true if load was successful
function ShakedownAddonBase.prototype:Load()
    self:Debug("ShakedownAddonBase:Load")
    
    if self.loaded then return true end

    -- attempt to load
    local loaded, _ = LoadAddOn(self.addonName)
    
    -- did we load OK?
    self.loaded = (loaded == 1)
    
    if not self.loaded then
        self.activated = false
    end
    
    return (self.loaded)

end

function ShakedownAddonBase.prototype:Activate()
    self:Debug("Activating shakedown for "..tostring(self))
    
    self.activated = true
    
    -- if it isn't loaded, attempt to load it first
    return (self.loaded or self:Load())
    
end

function ShakedownAddonBase.prototype:Deactivate()
    self:Debug("Deactivating shakedown for "..tostring(self))
    self.activated = false
end

-- output the name of the addon on tostring calls
function ShakedownAddonBase.prototype:ToString()
    return self.displayName
end

-- compare function, so that we can sort by addon name
function ShakedownAddonBase.prototype:IsLessThan(other)
    return (self.displayName < other.displayName)
end

-- turn debugging on or off
function ShakedownAddonBase.prototype:SetDebug(enabled)
    self:SetDebugging(enabled)
end
