-- constants
local BAR_NUMBER = 5
local BAR_DURATION = { 10, 15, 20, 24, 27 }
local BAR_FREQUENCY = 30
local MSG_FREQUENCY = 3
local BAR_ID = "Shakedown BigWigs %d"

-- create the class
local AceOO = AceLibrary("AceOO-2.0")
local Addon = AceOO.Class(ShakedownAddonBase, "AceEvent-2.0")

-- members
Addon.prototype.testPlugin = nil
Addon.prototype.proximityPlugin = nil

-- constructor
function Addon.prototype:init()
    Addon.super.prototype.init(self, "BigWigs", "BigWigs")
end

function Addon.prototype:Activate()

    if not Addon.super.prototype.Activate(self) then return end

    -- activate the addon if necessary
    if not BigWigs:IsActive() then
        BigWigs:ToggleActive(true)
    end
    
    -- activate the test plugin
    if not self.testPlugin then
        self.testPlugin = BigWigs:GetModule("Test")
    end
    if not self.testPlugin then return end
    
    -- attempt to activate the proximity plugin
    if not self.proximityPlugin then
        self.proximityPlugin = BigWigs:GetModule("Proximity")
    end
    
    -- show a message now
    self:ShowMessage()
    
    -- show bars now
    self:ShowBar()

    --schedule repeating messages and bars
    self:ScheduleRepeatingEvent("BigWigsShakedownMessage", self.ShowMessage, MSG_FREQUENCY, self)
    self:ScheduleRepeatingEvent("BigWigsShakedownBars", self.ShowBar, BAR_FREQUENCY, self)
    
    -- show the bar anchors
    BigWigs.cmdtable.args.Plugins.args.Bars.args.anchor.set("anchor", true)
    
    -- show the message anchor
    if not BigWigs.cmdtable.args.Plugins.args.Messages.args.anchor.disabled() then
        BigWigs.cmdtable.args.Plugins.args.Messages.set("anchor", true)
    end
    
    -- show the proximity frame
    if self.proximityPlugin then
        self.proximityPlugin:BigWigs_ShowProximity(self)
    end
end

function Addon.prototype:Deactivate()

    Addon.super.prototype.Deactivate(self)

    -- check existence
    if not BigWigs then return end
    if not self.testPlugin then return end
    
    -- kill the scheduled messages and bars
    self:CancelScheduledEvent("BigWigsShakedownMessage")
    self:CancelScheduledEvent("BigWigsShakedownBars")
    
    -- kill each of the bars
    for i = 1, BAR_NUMBER do
        self.testPlugin:TriggerEvent("BigWigs_StopBar", self.testPlugin, string.format(BAR_ID, i))
    end
    
    -- hide the bar anchors
    BigWigs.cmdtable.args.Plugins.args.Bars.args.anchor.set("anchor", false)

    -- hide the message anchor
    BigWigs.cmdtable.args.Plugins.args.Messages.set("anchor", false)
    
    -- hide the proximity frame
    if self.proximityPlugin then
        self.proximityPlugin:BigWigs_HideProximity(self)
    end
end

-- create some bars
function Addon.prototype:ShowBar()

    if not self.testPlugin then return end

    for i = 1, BAR_NUMBER do
        self.testPlugin:Bar( string.format(BAR_ID, i), BAR_DURATION[i], "Spell_Nature_ResistNature")
    end

end

-- show a message
function Addon.prototype:ShowMessage()

    if not self.testPlugin then return end
   
    -- display a single message
    self.testPlugin:TriggerEvent("BigWigs_Message", "BigWigs Message", "Important", true)
    
end

-- dummy proximity check for the proximity frame to use
Addon.prototype.proximityCheck = function( unit ) 
	return false
end
Addon.prototype.db = { profile = { proximity = true } }

-- create the instance of this class
local thisAddon = Addon:new()