-- localisations
local L = AceLibrary("AceLocale-2.2"):new("NShakedown")

L:RegisterTranslations("enUS", function() return {
    -- options
    ["Start"] = true,
    ["Start shakedown."] = true,
    ["Stop"] = true,
    ["Stop shakedown."] = true,
    ["Addons"] = true,
    ["Enable/disable shakedown of Addons."] = true,
    ["Enable/disable shakedown of %s."] = true,
    ["Enable All"] = true,
    ["Enable shakedown for all Addons."] = true,
    ["Disable All"] = true,
    ["Disable shakedown for all Addons."] = true,
    
    -- keybinging
    ["Start/stop a shakedown."] = true,
    
	-- fubar
    ["FuBarPlugin Config"] = true,
    ["Configure the FuBar Plugin."] = true,
    ["|cffffff00Click|r to %s shakedown."] = true,
    ["|cffffff00Right-Click|r to change settings."] = true,
} end)

-- constants
local ICON = "Interface\\Addons\\NShakedown\\Icons\\shakedown"
local ICON_ROT = "Interface\\Addons\\NShakedown\\Icons\\shakedown_rot"
local TIMER_FREQ = 0.2
local TIMER_EVENT = "SD_CHANGE_ICON"

-- create the class
local VERSION = tonumber(("$Revision: 42321 $"):match("%d+"))
NShakedown = AceLibrary("AceAddon-2.0"):new("AceConsole-2.0", "AceDB-2.0", "AceDebug-2.0", "AceEvent-2.0", "FuBarPlugin-2.0")
NShakedown.version = "r" .. VERSION
NShakedown.revision = VERSION
NShakedown.date = ("$Date: 2007-06-30 06:41:37 -0400 (Sat, 30 Jun 2007) $"):match("%d%d%d%d%-%d%d%-%d%d")

-- members
NShakedown.activated = false
NShakedown.enabled = false
NShakedown.rotatedIcon = false

function NShakedown:OnInitialize()

    -- register the database, with defaults
    self:RegisterDB("NShakedownDB")
    self:RegisterDefaults("profile", {
        debug = false,
        state = {}
    })
    
    -- enable/disable debugging
    self:SetDebugging(self.db.profile.debug)
    self:Debug("NShakedown:OnInitialize")
    self:SetAddonDebug()
    
    -- allow slash commands
    self:RegisterChatCommand({"/shakedown", "/nshakedown"}, self.options)

    -- register the keybindings
    BINDING_HEADER_NSHAKEDOWN = "NShakedown"
    BINDING_NAME_NSHAKEDOWN_START = L["Start/stop a shakedown."]
    
    -- sort the addons alphabetically
    table.sort(SDAddons)
    
    local i = 101
    local opt = self.OnMenuRequest.args
    for k, v in ipairs(SDAddons) do
        displayName = tostring(v)
        
        -- default to activated
        if NShakedown.db.profile.state[displayName] == nil then
            self:Debug(" defaulting new addon "..displayName.." to enabled")
            NShakedown.db.profile.state[displayName] = true
        end
        
        -- activate according to the profile
        v.enabled = NShakedown.db.profile.state[displayName]
        
        -- populate the menu
        if v.addonName ~= "StandardUI" then
            opt.enable.args[k] = {
                type = "toggle",
                name = displayName,
                desc = L["Enable/disable shakedown of %s."]:format(displayName),
                get = function()
                    return v.enabled
                end,
                set = function(value)
                    NShakedown.db.profile.state[tostring(v)] = value
                    v.enabled = value
                end,
                order = i,
            }
        end
    end
    
end

function NShakedown:OnEnable(first)

    self:Debug("NShakedown:OnEnable first = "..tostring(first))
    self.enabled = true

end

function NShakedown:OnDisable()

    self:Debug("NShakedown:OnDisable")
    self:Deactivate()
    self.enabled = false

end

function NShakedown:Activate()

    if not self.enabled then return end

    self:Debug("NShakedown:Activate")

    if self.activated then return end

    self.activated = true
    
    -- start the icon flipping
    self:ScheduleRepeatingEvent(TIMER_EVENT, self.ChangeIcon, TIMER_FREQ, self)
    
    --make the addons sing and dance
    for k, v in ipairs(SDAddons) do
        if v.enabled then
            v:Activate()
        end
    end
    
end

function NShakedown:Deactivate()

    self:Debug("NShakedown:Deactivate")

    --stop display
    for k, v in ipairs(SDAddons) do
        if v.activated then
            v:Deactivate()
        end
    end
    
    -- stop the icon flipping
    self:CancelScheduledEvent(TIMER_EVENT)
    self:SetIcon(ICON)
    self.rotatedIcon = false
    
    -- reset the menu
    self.OnMenuRequest.args.trigger.name = L["Start"]
    self.OnMenuRequest.args.trigger.desc = L["Start shakedown."]
   
    self.activated = false
end

function NShakedown:Trigger()

    if self.activated then
        self.OnMenuRequest.args.trigger.name = L["Start"]
        self.OnMenuRequest.args.trigger.desc = L["Start shakedown."]
        NShakedown:Deactivate()
    else
        self.OnMenuRequest.args.trigger.name = L["Stop"]
        self.OnMenuRequest.args.trigger.desc = L["Stop shakedown."]
        NShakedown:Activate()
    end
    
end

function NShakedown:OnDebugEnable()

    self:Debug("NShakedown:OnDebugEnable")
    
    self.db.profile.debug = true
    
    self:SetAddonDebug()
    
end

function NShakedown:OnDebugDisable()
    
    self:Debug("NShakedown:OnDebugDisable")
    
    self.db.profile.debug = false

    self:SetAddonDebug()
    
end

-- turn debugging on or off for each of the addons
function NShakedown:SetAddonDebug()

    self:Debug("NShakedown:SetAddonDebug self.db.profile.debug = "..tostring(self.db.profile.debug))

    --enable/disable for all addons
    for k, v in ipairs(SDAddons) do
        v:SetDebug(self.db.profile.debug)
    end

end

-- register options
NShakedown.options = {
    type = "group",
    args = {
        trigger = {
            name = L["Start"],
            desc = L["Start shakedown."],
            type = "execute",
            func = function()
                NShakedown:Trigger()
            end,
            order = 1,
        },
        enable = {
            name = L["Addons"],
            desc = L["Enable/disable shakedown of Addons."],
            type = 'group',
            order = 2,
            args = {
                spacer1 = {
                    type = 'header',
                    order = 200,
                },
                enable = {
                    name = L["Enable All"],
                    desc = L["Enable shakedown for all Addons."],
                    type = "execute",
                    func = function()
                        for k, v in pairs(NShakedown.options.args.enable.args) do
                            if v.type == "toggle" then
                                v.set(true)
                            end
                        end
                    end,
                    order = 201,
                },
                disable = {
                    name = L["Disable All"],
                    desc = L["Disable shakedown for all Addons."],
                    type = "execute",
                    func = function()
                        for k, v in pairs(NShakedown.options.args.enable.args) do
                            if v.type == "toggle" then
                                v.set(false)
                            end
                        end
                    end,
                    order = 202,
                }
            }
        }
    },
}

-- FuBar options
NShakedown.hasIcon = ICON
NShakedown.hasNoText = true
NShakedown.hideWithoutStandby = true
NShakedown.independentProfile = true
NShakedown.cannotDetachTooltip = true
NShakedown.blizzardTooltip = true
NShakedown.OnMenuRequest = NShakedown.options

-- left click the FuBar plugin / minimap icon
function NShakedown:OnClick(button)
    NShakedown:Trigger()
end

-- tooltip formatting
function NShakedown:OnTooltipUpdate()
    local action = ""
    
    if self.activated then
        action = L["Stop"]
    else
        action = L["Start"]
    end
    
	GameTooltip:AddLine("NShakedown")
	GameTooltip:AddLine(" ")
	GameTooltip:AddLine(L["|cffffff00Click|r to %s shakedown."]:format(action), 0.2, 1, 0.2)
	GameTooltip:AddLine(L["|cffffff00Right-Click|r to change settings."], 0.2, 1, 0.2)
end

-- swap the icon
function NShakedown:ChangeIcon()

    if self.rotatedIcon then
        self:SetIcon(ICON)
    else
        self:SetIcon(ICON_ROT)
    end
    
    self.rotatedIcon = not self.rotatedIcon
end
