-- localisations
local L = AceLibrary("AceLocale-2.2"):new("Addon")

L:RegisterTranslations("enUS", function() return {
    ["Current Zone"] = true,
    ["Current Sub Zone"] = true,
    ["Zone PVP Info"] = true,
    ["PVP Arena Info"] = true,
    ["Autofollow status"] = true,
    ["Raid Warning"] = true,
    ["UI Error"] = true,
    ["UI Message"] = true,
    ["Mirror Timer %d"] = true,
    ["Casting Bar"] = true,
    ["Quest Watch Frame"] = true,
    ["Durability"] = true,
    ["Standard WoW GUI"] = true,
} end)

-- constants
local MSG_FREQUENCY = 3

local AceOO = AceLibrary("AceOO-2.0")
Addon = AceOO.Class(ShakedownAddonBase, "AceEvent-2.0")

Addon.prototype.mirrorCoveringFrame = nil
Addon.prototype.castbarCoveringFrame = nil
Addon.prototype.questWatchCoveringFrame = nil
Addon.prototype.durabilityCoveringFrame = nil
Addon.prototype.oldPVPInfoTextString = ""
Addon.prototype.oldPVPArenaTextString = ""
Addon.prototype.showErrorMsg = false

-- constructor
function Addon.prototype:init()
    -- call the parent, explaining that this "addon" is already loaded
    Addon.super.prototype.init(self, "StandardUI", L["Standard WoW GUI"], true)
end

function Addon.prototype:Activate()

    if not Addon.super.prototype.Activate(self) then return end

    -- set / store text values
    ZoneTextString:SetText(L["Current Zone"])
    SubZoneTextString:SetText(L["Current Sub Zone"])
    self.oldPVPInfoTextString = PVPInfoTextString:GetText()
    PVPInfoTextString:SetText(L["Zone PVP Info"])
    self.oldPVPArenaTextString = PVPArenaTextString:GetText()
    PVPArenaTextString:SetText(L["PVP Arena Info"])
    AutoFollowStatusText:SetText(L["Autofollow status"])
    
    -- show auto follow
    AutoFollowStatus:Show()
    
    -- set the frames to display for 5 mins
    FadingFrame_SetHoldTime(ZoneTextFrame, 300)
    FadingFrame_SetHoldTime(SubZoneTextFrame, 300)
    FadingFrame_SetHoldTime(AutoFollowStatus, 300)
    
    -- show the text frames
    FadingFrame_Show(ZoneTextFrame)
    FadingFrame_Show(SubZoneTextFrame)
    FadingFrame_Show(AutoFollowStatus)
    
    -- create a frame covering the mirror bars
    if not self.mirrorCoveringFrame then
        self.mirrorCoveringFrame = { nil, nil, nil }
        for i = 1, 3 do
            self.mirrorCoveringFrame[i] = CreateFrame("Frame", "mirrorCoveringFrame"..tostring(i), UIParent)
            self.mirrorCoveringFrame[i]:SetBackdrop({ bgFile = "Interface/Tooltips/UI-Tooltip-Background", })
            self.mirrorCoveringFrame[i]:SetPoint("TOPLEFT", getglobal("MirrorTimer"..tostring(i).."StatusBar"), "TOPLEFT", 0, 0)
            self.mirrorCoveringFrame[i]:SetPoint("BOTTOMRIGHT", getglobal("MirrorTimer"..tostring(i).."StatusBar"), "BOTTOMRIGHT", 0, 0)
            local label = self.mirrorCoveringFrame[i]:CreateFontString("mirrorCoveringFrame_label", "BORDER")
            label:SetPoint("TOPLEFT", self.mirrorCoveringFrame[i], "TOPLEFT", 0, 0)
            label:SetFont(NumberFontNormal:GetFont(), 12, "OUTLINE")
            label:SetText(L["Mirror Timer %d"]:format(i))
        end
    else
        for i = 1, 3 do
            self.mirrorCoveringFrame[i]:Show()
        end
    end
    
    -- create a frame covering the cast bar
    if not self.castbarCoveringFrame then
        self.castbarCoveringFrame = CreateFrame("Frame", "castbarCoveringFrame", UIParent)
        self.castbarCoveringFrame:SetBackdrop({ bgFile = "Interface/Tooltips/UI-Tooltip-Background", })
        self.castbarCoveringFrame:SetPoint("TOPLEFT", CastingBarFrame, "TOPLEFT", 0, 0)
        self.castbarCoveringFrame:SetPoint("BOTTOMRIGHT", CastingBarFrame, "BOTTOMRIGHT", 0, 0)
        local label = self.castbarCoveringFrame:CreateFontString("castbarCoveringFrame_label", "BORDER")
        label:SetPoint("TOPLEFT", self.castbarCoveringFrame, "TOPLEFT", 0, 0)
        label:SetFont(NumberFontNormal:GetFont(), 12, "OUTLINE")
        label:SetText(L["Casting Bar"])
    else
        self.castbarCoveringFrame:Show()
    end
    
    -- create a frame covering the quest watch
    if not self.questWatchCoveringFrame then
        self.questWatchCoveringFrame = CreateFrame("Frame", "questWatchCoveringFrame", UIParent)
        self.questWatchCoveringFrame:SetBackdrop({ bgFile = "Interface/Tooltips/UI-Tooltip-Background", })
        self.questWatchCoveringFrame:SetPoint("TOPRIGHT", QuestWatchFrame, "TOPRIGHT", 0, 0)
        self.questWatchCoveringFrame:SetWidth(QuestWatchFrame:GetWidth())
        self.questWatchCoveringFrame:SetHeight(200)
        local label = self.questWatchCoveringFrame:CreateFontString("questWatchCoveringFrame_label", "BORDER")
        label:SetPoint("TOPLEFT", self.questWatchCoveringFrame, "TOPLEFT", 0, 0)
        label:SetFont(NumberFontNormal:GetFont(), 12, "OUTLINE")
        label:SetText(L["Quest Watch Frame"])
    else
        self.questWatchCoveringFrame:Show()
    end
    
    -- create a frame covering the durability indicator
    if not self.durabilityCoveringFrame then
        self.durabilityCoveringFrame = CreateFrame("Frame", "durabilityCoveringFrame", UIParent)
        self.durabilityCoveringFrame:SetBackdrop({ bgFile = "Interface/Tooltips/UI-Tooltip-Background", })
        self.durabilityCoveringFrame:SetPoint("TOPLEFT", DurabilityFrame, "TOPLEFT", 0, 0)
        self.durabilityCoveringFrame:SetPoint("BOTTOMRIGHT", DurabilityFrame, "BOTTOMRIGHT", 0, 0)
        local label = self.durabilityCoveringFrame:CreateFontString("durabilityCoveringFrame_label", "BORDER")
        label:SetPoint("TOPLEFT", self.durabilityCoveringFrame, "TOPLEFT", 0, 0)
        label:SetFont(NumberFontNormal:GetFont(), 12, "OUTLINE")
        label:SetText(L["Durability"])
    else
        self.durabilityCoveringFrame:Show()
    end
    
    -- show a message now
    self:ShowMessage()
    
    -- schedule repeating messages
    self:ScheduleRepeatingEvent("StandardUIShakedown", self.ShowMessage, MSG_FREQUENCY, self)

end

function Addon.prototype:Deactivate()

    Addon.super.prototype.Deactivate(self)

    -- hide the covering frames
    for i = 1, 3 do
        self.mirrorCoveringFrame[i]:Hide()
    end
    self.castbarCoveringFrame:Hide()
    self.questWatchCoveringFrame:Hide()
    self.durabilityCoveringFrame:Hide()
    
    -- hide auto follow
    AutoFollowStatus:Hide()

    -- reset the old values
    FadingFrame_SetHoldTime(ZoneTextFrame, 0)
    FadingFrame_SetHoldTime(SubZoneTextFrame, 0)
    FadingFrame_SetHoldTime(AutoFollowStatus, 0)
    PVPInfoTextString:SetText(self.oldPVPInfoTextString)
    PVPArenaTextString:SetText(self.oldPVPArenaTextString)
    
    -- hide the raid warning frame
    RaidWarningFrame:Clear()

    self:CancelScheduledEvent("StandardUIShakedown")
    
end

-- show a message in the errors frame
function Addon.prototype:ShowMessage()

    -- UI errors/messages
    if self.showErrorMsg then
        UIErrorsFrame:AddMessage(L["UI Error"], 1.0, 0.1, 0.1, 1.0)
    else
        UIErrorsFrame:AddMessage(L["UI Message"], NORMAL_FONT_COLOR.r, NORMAL_FONT_COLOR.g, NORMAL_FONT_COLOR.b, 1.0)
    end
    self.showErrorMsg = not self.showErrorMsg
    
    -- raid warning
    RaidWarningFrame:AddMessage(L["Raid Warning"], 1, 0, 0, 1, 1)

end

-- create the instance of this class
local thisAddon = Addon:new()
